# Python mail Google

![category](python)

## Only SMTP

Jusqu'à présent il était assez simple d'envoyer un mail par le serveur **smtp** de gmail en python

Exemple d'envoi d'un message basique

```python
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.message import EmailMessage
myuser = 'myem@il.fr'
mypassword ='*****'
to = ['tutu@tutu.fr', 'toto@toto.fr']
Toadd = ",".join(to)
bcc = "copy@cached.fr"
message = EmailMessage()
message['From'] = myuser
message['To'] = Toadd
message['BCC'] = bcc
message['Subject'] = "my title"
message.set_content("my text ....")
serveur = smtplib.SMTP('smtp.gmail.com', 587)
serveur.starttls()
serveur.login(myuser, mypassword)
serveur.send_message(message)
serveur.quit()
```

Si on veut ajouter des pièces jointes

```python
import mimetypes

path='/home/user/test.pdf'
filename = 'test.pdf'
ctype, encoding = mimetypes.guess_type(path)
if ctype is None or encoding is None:
    ctype = 'application/octet-stream'
maintype, subtype = ctype.split('/', 1)
with open(path, 'rb') as fp:
    message.add_attachment(fp.read(),
        maintype=maintype,
        subtype=subtype,
        filename=filename)
```

## with Oauth2

Google impose maintenant l'usage de Oauth2
Pour cela il va falloir aller sur https://console.developers.google.com/ et créer un projet

![](https://lh4.googleusercontent.com/cQ3I0lmHvuA46MI7Btd3fL-4AVbcyFE6Z_a6OA3l3XrN6jsETLZNzt-uvrfu9wfiTN-BmL3EX2RLQapRZ1ILEmjfFB_BhcunrubAgBhRSJCmlXtg2Cv1toNKm7lznukYB0LgjYbP)

Ajouter dans le projet les API Gmail via l'onglet librairie

![](http://https://lh6.googleusercontent.com/qRA82H-dl5bKqCzC8H_YpR8cDa23K85f6s5Q2piOw95VO62mqn_YfOkLFVaSD_p0qCVKTLGRLeFDMzU3Ad-Lb8Ou0Yde3efa7diXO5QDVC7IjCQcrbQh-H30xBVpy24KfrSZJ1Uc)

Par la suite il faut créer un credentials oauth2 et un écran de consentement

![](https://lh6.googleusercontent.com/UwsT80hMiX5B1TjEg51gbqkkJ-iembuOfCu3cM9k1pdcv2mlleKht4k_FmeRFBwFApyuHkqb8Xgxut9dHybJAIQCtzfvul60Uh957dPXi3fRFrsU4ZjhTSZo_-8byEsxIy52jSRv)

![](http://https://lh5.googleusercontent.com/WAhpqm_Hwa640iUPpKK7eWRPDIPf2FlpQ7Y5XT2uKa9UrfO-i868IpL-O4orLMXSMT6wRnubrYzyZD0LRUSjKt9xJDSOdZSrZmjBTuDKj3w9pGHG2U_m_afZ8RlN9R2SXmDHcXht)

A partir de là on peut télécharger via l'onglet identifiant le client Oauth qu'on doit renommer "credentials.json"

A partir de maintenant il va falloir créer un token et envoyer des mails

Tout d'abord il faut installer les librairies google de connexion

```
pip install --upgrade google-api-python-client google-auth-httplib2 google-auth-oauthlib
```

On peut utiliser pour tester la connexion le code quickstart.py disponible sur https://github.com/googleworkspace/python-samples/tree/master/gmail/quickstart

```python
# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# [START gmail_quickstart]
from __future__ import print_function

import os.path

from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow
from googleapiclient.discovery import build
from googleapiclient.errors import HttpError

# If modifying these scopes, delete the file token.json.
SCOPES = ['https://www.googleapis.com/auth/gmail.readonly']


def main():
    """Shows basic usage of the Gmail API.
    Lists the user's Gmail labels.
    """
    creds = None
    # The file token.json stores the user's access and refresh tokens, and is
    # created automatically when the authorization flow completes for the first
    # time.
    if os.path.exists('token.json'):
        creds = Credentials.from_authorized_user_file('token.json', SCOPES)
    # If there are no (valid) credentials available, let the user log in.
    if not creds or not creds.valid:
        if creds and creds.expired and creds.refresh_token:
            creds.refresh(Request())
        else:
            flow = InstalledAppFlow.from_client_secrets_file(
                'credentials.json', SCOPES)
            creds = flow.run_local_server(port=0)
        # Save the credentials for the next run
        with open('token.json', 'w') as token:
            token.write(creds.to_json())

    try:
        # Call the Gmail API
        service = build('gmail', 'v1', credentials=creds)
        results = service.users().labels().list(userId='me').execute()
        labels = results.get('labels', [])

        if not labels:
            print('No labels found.')
            return
        print('Labels:')
        for label in labels:
            print(label['name'])

    except HttpError as error:
        # TODO(developer) - Handle errors from gmail API.
        print(f'An error occurred: {error}')


if __name__ == '__main__':
    main()
```

le lancement de ce code va créer un fichier token.json et une demande de validation

SI on veut envoyer des emails on peut utiliser les codes sur https://github.com/googleworkspace/python-samples/tree/master/gmail/snippet/send%20mail

mais avant il faut supprimer le token.json car il a été créer un avec scope de "readonly" alors qu'on va vouloir envoyer des mails

Exemple de code

```python
# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# [START gmail_quickstart]
from __future__ import print_function

import os.path

from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow
from googleapiclient.discovery import build
from googleapiclient.errors import HttpError


import base64
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart

# If modifying these scopes, delete the file token.json.
#SCOPES = ['https://www.googleapis.com/auth/gmail.readonly']
SCOPES = ['https://www.googleapis.com/auth/gmail.send']

def CreateMessage(sender, to, subject, message_text):
    msg = MIMEMultipart('alternative')
    msg['Subject'] = subject
    msg['From'] = sender
    msg['To'] = to
    msg.attach(MIMEText(message_text, 'plain'))
    return {'raw': base64.urlsafe_b64encode(msg.as_string().encode()).decode()}


def gmail_send_message(creds):
    """Create and send an email message
    Print the returned  message id
    Returns: Message object, including message id
    Load pre-authorized user credentials from the environment.
    TODO(developer) - See https://developers.google.com/identity
    for guides on implementing OAuth2 for the application.
    """

    try:
        service = build('gmail', 'v1', credentials=creds)
        message = CreateMessage('fraoustin@gmail.com', 'fraoustin@gmail.com', 'Test mail', 'Sample of email')
        send_message = (service.users().messages().send(userId="me", body=message).execute())
        print(F'Message Id: {send_message["id"]}')
    except HttpError as error:
        print(F'An error occurred: {error}')
        send_message = None
    return send_message

def main():
    """Shows basic usage of the Gmail API.
    Lists the user's Gmail labels.
    """
    creds = None
    # The file token.json stores the user's access and refresh tokens, and is
    # created automatically when the authorization flow completes for the first
    # time.
    if os.path.exists('token.json'):
        creds = Credentials.from_authorized_user_file('token.json', SCOPES)
    # If there are no (valid) credentials available, let the user log in.
    if not creds or not creds.valid:
        if creds and creds.expired and creds.refresh_token:
            creds.refresh(Request())
        else:
            flow = InstalledAppFlow.from_client_secrets_file(
                'credentials.json', SCOPES)
            creds = flow.run_local_server(port=0)
        # Save the credentials for the next run
        with open('token.json', 'w') as token:
            token.write(creds.to_json())

    try:
        gmail_send_message(creds)
    except HttpError as error:
        print(f'An error occurred: {error}')


if __name__ == '__main__':
    main()
```

